/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.servlets.format.sideboxes;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import com.silverwrist.util.DOMElementHelper;
import com.silverwrist.util.StringUtil;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.servlets.format.*;

public class ConferenceBox implements SideBoxFactory
{
  /*--------------------------------------------------------------------------------
   * Internal class that implements the actual sidebox
   *--------------------------------------------------------------------------------
   */

  class ConferenceBoxImpl implements ContentRender, ColorSelectors
  {
    private UserContext uc;     // current user context
    private List hotlist;       // current conference hotlist

    ConferenceBoxImpl(UserContext uc) throws DataException
    {
      this.uc = uc;
      this.hotlist = uc.getConferenceHotlist();

    } // end constructor

    public String getPageTitle(RenderData rdat)
    {
      if (uc.isLoggedIn())
	return title;
      else
	return anon_title;

    } // end getPageTitle

    public String getPageQID()
    {
      return null;

    } // end getPageQID

    public void renderHere(Writer out, RenderData rdat) throws IOException
    {
      String norm_font = rdat.getStdFontTag(SIDEBOX_CONTENT_FOREGROUND,2);
      if (hotlist.size()>0)
      { // write out the hotlist....
	String hilite = rdat.getStdFontTag(SIDEBOX_CONTENT_LINK,2);
	out.write("<TABLE ALIGN=CENTER BORDER=0 CELLPADDING=0 CELLSPACING=2>\n");
	Iterator it = hotlist.iterator();
	while (it.hasNext())
	{ // get the next hotlist entry and add it to the display
	  ConferenceHotlistEntry hle = (ConferenceHotlistEntry)(it.next());
	  ConferenceContext conf = hle.getConference();
	  String href = "confdisp?sig=" + conf.getEnclosingSIG().getSIGID() + "&conf=" + conf.getConfID();
	  out.write("<TR VALIGN=MIDDLE>\n<TD ALIGN=CENTER WIDTH=" + bullet_width + ">");
	  renderBullet(out,rdat);
	  out.write("</TD>\n<TD ALIGN=LEFT CLASS=\"sidebox\">\n" + norm_font
		    + "<B><A CLASS=\"sidebox\" HREF=\"" + rdat.getEncodedServletPath(href) + "\">" + hilite
		    + StringUtil.encodeHTML(conf.getName()) + "</FONT></A></B> ("
		    + StringUtil.encodeHTML(conf.getEnclosingSIG().getName()) + ")</FONT>\n");
	  if (conf.anyUnread())
	  { // write out the new-messages tag and its enclosing link
	    out.write("&nbsp;<A HREF=\"" + rdat.getEncodedServletPath(href + "&rnm=1") + "\">");
	    renderNewMessageTag(out,rdat);
	    out.write("</A>\n");

	  } // end if

	  out.write("</TD>\n</TR>\n");

	} // end while

	out.write("</TABLE>\n");

      } // end if
      else // write the "null" message
	out.write(norm_font + "<EM>" + null_message + "</EM></FONT>\n");

      if (uc.isLoggedIn())
      { // write the link at the end
	out.write("<P>" + rdat.getStdFontTag(SIDEBOX_CONTENT_FOREGROUND,1)
		  + "<B>[ <A CLASS=\"sidebox\" HREF=\"" + rdat.getEncodedServletPath("settings?cmd=H") + "\">"
		  + rdat.getStdFontTag(SIDEBOX_CONTENT_LINK,1) + manage_link + "</FONT></A> ]</B></FONT>");

      } // end if

    } // end renderHere

  } // end class ConferenceBoxImpl

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private String title;                   // title of sidebox for logged-in users
  private String anon_title;              // title of sidebox for not-logged-in users
  private String null_message;            // message if they have no hotlist conferences
  private String manage_link;             // "Manage" link text
  private String bullet_url;              // URL to bullet image
  private boolean bullet_fixup;           // does bullet image need to be run through getFullImagePath?
  private int bullet_width;               // width of bullet image
  private int bullet_height;              // height of bullet image
  private String newmsg_url;              // URL to new messages tag image
  private boolean newmsg_fixup;           // does new messages image need to be run through getFullImagePath?
  private String newmsg_alt;              // ALT text to use for new message tag image
  private int newmsg_width;               // width of new message tag image
  private int newmsg_height;              // height of new message tag image

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  public ConferenceBox()
  { // do nothing
  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  private void renderBullet(Writer out, RenderData rdat) throws IOException
  {
    synchronized (this)
    { // this may be called by more than one class, so protect this part
      if (bullet_fixup)
      { // fix up the bullet URL
	bullet_url = rdat.getFullImagePath(bullet_url);
	bullet_fixup = false;

      } // end if

    } // end synchronized block

    out.write("<IMG SRC=\"" + bullet_url + "\" ALT=\"*\" WIDTH=" + bullet_width + " HEIGHT=" + bullet_height
	      + " BORDER=0>");

  } // end renderBullet

  private void renderNewMessageTag(Writer out, RenderData rdat) throws IOException
  {
    synchronized (this)
    { // this may be called by more than one class, so protect this part
      if (newmsg_fixup)
      { // fix up the host tag URL
	newmsg_url = rdat.getFullImagePath(newmsg_url);
	newmsg_fixup = false;

      } // end if

    } // end synchronized block

    out.write("<IMG SRC=\"" + newmsg_url + "\" ALT=\"" + newmsg_alt + "\" WIDTH=" + newmsg_width + " HEIGHT="
	      + newmsg_height + " BORDER=0>");

  } // end renderNewMessageTag

  /*--------------------------------------------------------------------------------
   * Implementations from interface SideBoxFactory
   *--------------------------------------------------------------------------------
   */

  public void setConfiguration(Element cfg) throws ConfigException
  {
    DOMElementHelper cfg_h = new DOMElementHelper(cfg);
    title = cfg_h.getSubElementText("title");
    if (title==null)
      throw new ConfigException("no <title/> specified for conference list sidebox",cfg);
    title += ":";
    anon_title = cfg_h.getSubElementText("anon-title");
    if (anon_title==null)
      anon_title = title;
    else
      anon_title += ":";

    null_message = cfg_h.getSubElementText("null-msg");
    if (null_message==null)
      null_message = "You are not a member of any SIGs.";

    manage_link = cfg_h.getSubElementText("manage-link");
    if (manage_link==null)
      manage_link = "Manage";

    Element x = cfg_h.getSubElement("bullet");
    if (x!=null)
    { // get the parameters for the bullet image
      DOMElementHelper x_h = new DOMElementHelper(x);
      bullet_url = x_h.getElementText();
      if (StringUtil.isStringEmpty(bullet_url))
      { // default the bullet URL and fixup parameter
	bullet_url = "purple-ball.gif";
	bullet_fixup = true;

      } // end if
      else  // get the "fixup" parameter
	bullet_fixup = x_h.hasAttribute("fixup");
      
      Integer xi = x_h.getAttributeInt("width");
      bullet_width = (xi!=null) ? xi.intValue() : 14;
      xi = x_h.getAttributeInt("height");
      bullet_height = (xi!=null) ? xi.intValue() : 14;

    } // end if
    else
    { // just default all the bullet parameters
      bullet_url = "purple-ball.gif";
      bullet_fixup = true;
      bullet_width = 14;
      bullet_height = 14;

    } // end else

    x = cfg_h.getSubElement("new-image");
    if (x!=null)
    { // get the parameters for the host tag image
      DOMElementHelper x_h = new DOMElementHelper(x);
      newmsg_url = x_h.getElementText();
      if (StringUtil.isStringEmpty(newmsg_url))
      { // default the host tag image URL and fixup parameter
	newmsg_url = "tag_new.gif";
	newmsg_fixup = true;

      } // end if
      else  // get the "fixup" parameter
	newmsg_fixup = x_h.hasAttribute("fixup");

      newmsg_alt = x.getAttribute("alt");
      if (StringUtil.isStringEmpty(newmsg_alt))
	newmsg_alt = "New!";
      
      Integer xi = x_h.getAttributeInt("width");
      newmsg_width = (xi!=null) ? xi.intValue() : 40;
      xi = x_h.getAttributeInt("height");
      newmsg_height = (xi!=null) ? xi.intValue() : 20;

    } // end if
    else
    { // just default all the host tag parameters
      newmsg_url = "tag_new.gif";
      newmsg_fixup = true;
      newmsg_alt = "New!";
      newmsg_width = 40;
      newmsg_height = 20;

    } // end else

  } // end setConfiguration

  public VeniceContent create(VeniceEngine engine, UserContext uc) throws AccessError, DataException
  {
    return new ConferenceBoxImpl(uc);

  } // end create

} // end class ConferenceBox
